"""simple interface for demonstration"""

import argparse
import re
from greedy_alg import greedy
try:
    import pytest
except ImportError:
    pytest = None


def main():
    """CLI arguments, possible options for finding set of operators"""
    parser = argparse.ArgumentParser()
    parser.add_argument('-cover', type=str, help='A dictionary whose keys are the names of mobile operators, '
                                                 'and the values are the sets of regions they cover', default='')
    parser.add_argument('-regions', type=str, help='The set of all regions that need to be covered', default='')
    parser.add_argument('-t', '--test', action='store_true', help='test mode')
    args = parser.parse_args()
    phone_operator_regions_str = args.cover
    reg = args.regions
    test = args.test
    if test:
        print('TEST')
        if pytest:
            pytest.main(['-q', 'operators_greedy_alg/test.py'])
        else:
            print('No pytest found, only doctest will run')
    else:
        phone_operator_regions = dict()
        phone_operators = re.findall(r'[a-zA-Z]+', phone_operator_regions_str)
        regions_covered_mas = re.findall(r'\[[^a-zA-Z]+]', phone_operator_regions_str)
        regions_covered = []
        regions = set(map(int, reg[1:-1].split()))
        for i in range(len(regions_covered_mas)):
            regions_covered.append(set(map(int, regions_covered_mas[i][1:-1].split())))
        for j in range(len(phone_operators)):
            phone_operator_regions[phone_operators[j]] = regions_covered[j]
        print(greedy(phone_operator_regions, regions))


if __name__ == '__main__':
    main()



"""example module"""

from greedy_alg import greedy

print('example 1')
phone_operator_regions_1 = {'MTS': {4, 5, 7, 9},
                            'MEGAFON': {1, 2, 3, 4, 5, 6},
                            'YOTA': {1, 4, 5, 10, 11}}
regions_1 = {1, 2, 3, 4, 5, 7, 11}
print(phone_operator_regions_1, regions_1,)
print(greedy(phone_operator_regions_1, regions_1))

print('example 2')
phone_operator_regions_2 = {'MTS': {4, 5, 7, 9},
                            'MEGAFON': {1, 2, 3, 4, 5, 6},
                            'YOTA': {1, 4, 5, 10, 11}}
regions_2 = {1, 2, 3, 4, 5, 7, 77}
print(phone_operator_regions_2, regions_2)
print(greedy(phone_operator_regions_2, regions_2))

print('example 3')
phone_operator_regions_3 = {'YOTA': {1, 5, 19, 77, 1001},
                            'MTS': {2, 3, 4, 5, 6, 7},
                            'MEGAFON': {1, 3, 5, 177}}
regions_3 = {2, 5, 6, 7, 77, 177, 1001}
print(phone_operator_regions_3, regions_3)
print(greedy(phone_operator_regions_3, regions_3))



"""The module "greedy alg"
a) select the operator covering the largest number
of regions and not yet included in the coverage. If the operator will cover
some regions already included in the coverage, this is normal;
b) repeat until uncovered elements of the set remain"""


def greedy(phone_operator_regions, regions):
    """
        module example:
        >>> greedy({'MTS': {4, 5, 7, 9}, 'MEGAFON': {1, 2, 3, 4, 5, 6}}, {1, 2, 3, 4, 5, 7})
        ['MEGAFON', 'MTS']
        >>> greedy({'MTS': {4, 5, 7, 9}, 'MEGAFON': {1, 2, 3, 4, 5, 6}}, {1, 2, 3, 4, 5, 1001})
        'Один из регионов не входит в покрытие всех операторов'
        :param phone_operator_regions: dictionary with operators
        :param regions: set of regions
        :return: list with answer
        """
    phone_operator = list(phone_operator_regions)
    answer = []
    while regions != set():
        hit_max = name_of_max = 0
        for i in range(len(phone_operator)):
            hits = len(phone_operator_regions[phone_operator[i]] & regions)
            if hits > hit_max:
                hit_max = hits
                name_of_max = phone_operator[i]
        try:
            regions = regions.difference(phone_operator_regions[name_of_max])
            del phone_operator_regions[name_of_max]
            phone_operator.remove(name_of_max)
            answer.append(name_of_max)
        except KeyError:
            print('Один из регионов не входит в покрытие всех операторов')
            return ''
    return answer


if __name__ == '__main__':
    import doctest
    doctest.testmod()



from os import path
from setuptools import setup, find_packages

try:
    current_path = path.abspath(path.dirname(__file__))
except NameError:
    current_path = None

try:
    with open(path.join(current_path, 'README.md')) as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = ''

setup(
    name='operators_greedy_alg',
    version='1.1',
    license='MIT License',
    author="Ramil Akhmadullin",
    author_email='ramram110804@gmail.com',
    description='select the operator covering the largest number of regions',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    python_requires='>=3.5',
    install_requires=[],
    entry_points={
        'console_scripts': [
            'main=operators_greedy_alg:main',
        ],
    },
)